package hr.fer.zemris.pso;

/**
 * Razred implementira pojam globalnog susjedstva. Kod ove
 * vrste susjedstva, svaka čestica svjesna je najboljeg rješenja
 * koje je pronašla cjelokupna populacija.
 * 
 * @author marcupic
 */
public class GlobalNeighborhood implements Neighborhood {

	// Broj čestica
	int particlesCount;

	// Dimezija prostora
	int dims;
	
	// Najbolje globalno rješenje
	double[] best;

	// Radi li se minimizacija (true) ili maksimizacija (false)
	boolean minimize;
	
	/**
	 * Konstruktor susjedstava.
	 * 
	 * @param particlesCount broj čestica
	 * @param dims dimenzija
	 * @param minimize true ako se radi minimizacija, false inače
	 */
	public GlobalNeighborhood(int particlesCount, int dims, boolean minimize) {
		this.particlesCount = particlesCount;
		this.dims = dims;
		this.minimize = minimize;
		best = new double[dims];
	}
	
	/**
	 * Pronalazi globalno najbolje rješenje populacije.
	 * 
	 * @see hr.fer.zemris.pso.Neighborhood#scan(hr.fer.zemris.pso.Particle[])
	 */
	@Override
	public void scan(Particle[] particles) {
		int bestindex = 0;
		double bestValue = particles[bestindex].bestValue;
		for(int i = 1; i < particles.length; i++) {
			if((minimize && particles[i].bestValue<bestValue) || (!minimize && particles[i].bestValue>bestValue)) {
				bestValue = particles[i].bestValue;
				bestindex = i;
			}
		}
		for(int d = 0; d < dims; d++) {
			best[d] = particles[bestindex].bestVars[d];
		}
	}
	
	/**
	 * Vraća najbolje rješenje za zadanu česticu.
	 * @see hr.fer.zemris.pso.Neighborhood#findBest(int)
	 */
	@Override
	public double[] findBest(int forIndex) {
		return best;
	}
}
