package hr.fer.zemris.pso;

/**
 * Razred implementira pojam lokalnog susjedstva određene širine.
 * 
 * @author marcupic
 */
public class LocalNeighborhood implements Neighborhood {

	// Broj čestica
	int particlesCount;

	// Dimezija prostora
	int dims;
	
	// Veličina susjedstva
	int nSize;
	
	// Najbolja rješenje za susjedstvo svake čestice
	double[][] best;

	// Radi li se minimizacija (true) ili maksimizacija (false)
	boolean minimize;
	
	/**
	 * Konstruktor.
	 * 
	 * @param particlesCount broj čestica
	 * @param dims broj dimenzija
	 * @param nSize veličina susjedstva
	 * @param minimize true ako se radi minimizacija, false inače
	 */
	public LocalNeighborhood(int particlesCount, int dims, int nSize, boolean minimize) {
		this.particlesCount = particlesCount;
		this.dims = dims;
		this.minimize = minimize;
		this.nSize = nSize;
		best = new double[particlesCount][dims];
	}
	
	/**
	 * Pronalazi najbolja rješenja susjedstva za sve jedinke.
	 * 
	 * @see hr.fer.zemris.pso.Neighborhood#scan(hr.fer.zemris.pso.Particle[])
	 */
	@Override
	public void scan(Particle[] particles) {
		for(int index = 0; index < particles.length; index++) {
			int startFrom = index - nSize/2;
			int endAt = index + nSize/2;
			if(startFrom < 0) startFrom = 0;
			if(endAt >= particles.length) endAt = particles.length-1;

			int bestindex = startFrom;
			double bestValue = particles[bestindex].bestValue;
			for(int i = startFrom+1; i <= endAt; i++) {
				if((minimize && particles[i].bestValue<bestValue)||(!minimize && particles[i].bestValue>bestValue)) {
					bestValue = particles[i].bestValue;
					bestindex = i;
				}
			}
			for(int d = 0; d < dims; d++) {
				best[index][d] = particles[bestindex].bestVars[d];
			}
		}
	}
	
	/**
	 * Vraća najbolje rješenje za zadanu česticu.
	 * @see hr.fer.zemris.pso.Neighborhood#findBest(int)
	 */
	@Override
	public double[] findBest(int index) {
		return best[index];
	}
}
