package hr.fer.zemris.pso;

/**
 * Sučelje koje apstrahira pojam susjedstva.
 * Različite implementacije ponudit će konkretne
 * definicije susjedstva.
 * 
 * @author marcupic
 *
 */
public interface Neighborhood {
	
	/**
	 * Metoda koja se mora pozvati nad populacijom
	 * kako bi se napunili podaci o rješenjima iz
	 * susjedstva. Ovo mora biti napravljeno prije
	 * uporabe funkcije {@linkplain #findBest(int)}
	 * i svakako prije bilo kakvih izmjena u česticama.
	 * 
	 * @param particles populacija čestica
	 */
	void scan(Particle[] particles);
	
	/**
	 * Metoda koja za česticu određenu indeksom vraća
	 * poziciju najboljeg rješenja pronađenog u njezinom
	 * susjedstvu.
	 *  
	 * @param forIndex indeks čestice
	 * @return najbolje rješenje u susjedstvu te čestice
	 */
	double[] findBest(int forIndex);
}
