package hr.fer.zemris.tsp;

/**
 * Razred predstavlja jedan grad. Grad je određen svojim imenom te 
 * koordinatama X i Y.<br>
 * <i>Važno:</i> Varijable {@linkplain #x}, {@linkplain #y}
 * i {@linkplain #name} su javne kako bi se omogućio minimalni
 * "overhead" prilikom izvođenja evolucijskih algoritama. Ovo ima 
 * kao ružnu posljedicu da se neopreznim programiranjem vrijednosti
 * mogu mijenjati od bilo kuda, što može dovesti do pogrešnog rada
 * programa!
 * 
 * @author marcupic
 */
public class City {
	// Koordinata X
	public int x;
	// Koordinata Y
	public int y;
	// Naziv grada
	public String name;
	
	/**
	 * Konstruktor. Ime se postavlja na null.
	 * @param x x koordinata
	 * @param y y koordinata
	 */
	public City(int x, int y) {
		this(null, x, y);
	}

	/**
	 * Konstruktor.
	 * @param name ime grada
	 * @param x x koordinata
	 * @param y y koordinata
	 */
	public City(String name, int x, int y) {
		super();
		this.name = name;
		this.x = x;
		this.y = y;
	}

	@Override
	public String toString() {
		if(name!=null) {
			return name+" ("+x+","+y+")";
		} else {
			return "City at ("+x+","+y+")";
		}
	}
}
