package hr.fer.zemris.tsp;

import java.util.Arrays;

/**
 * Razred koji predstavlja jedno rješenje problema TSP.
 * Razred automatski nudi mogućnosti <i>pool</i>-anja
 * pomoću razreda {@linkplain TSPSolutionPool}.<br>
 * <i>Važno:</i> Varijable {@linkplain #cityIndexes}, 
 * {@linkplain #tourLength} i {@linkplain #next} su javne kako bi 
 * se omogućio minimalni "overhead" prilikom izvođenja evolucijskih 
 * algoritama. Ovo ima kao ružnu posljedicu da se neopreznim 
 * programiranjem vrijednosti mogu mijenjati od bilo kuda, što može 
 * dovesti do pogrešnog rada programa! 
 * 
 * @author marcupic
 */
public class TSPSolution {
	// Indeksi kojima treba obići gradove
	public int[] cityIndexes;
	// Ukupna duljina ture
	public double tourLength;
	// Sljedeće rješenje; koristi se uz razred TSPSolutionPool 
	public TSPSolution next;

	/**
	 * Konstruktor.
	 */
	public TSPSolution() {
	}
	
	/**
	 * Konstruktor.
	 */
	public TSPSolution(TSPSolution next) {
		this.next = next;
	}
	
	@Override
	public String toString() {
		return Arrays.toString(cityIndexes)+", len="+tourLength;
	}
}
