package hr.fer.zemris.tsp;

/**
 * Pomoćni razred koji služi za iznajmljivanje rješenja.
 * Uporaba ovog razreda preporuča se u slučaju kada algoritam
 * u petlji privremeno stvara veliku količinu novih rješenje
 * i potom ih odbacuje. Neprestana uporaba memorijskog alokatora
 * u takvom bi slučaju bila izuzetno neefikasna.
 * Umjesto toga, ovaj razred objekte iznajmljuje i stvara ih
 * po potrebi. Jednom kada je objekt stvoren, operacija
 * dohvata i vraćanja je o(1).
 *  
 * @author marcupic
 *
 */
public class TSPSolutionPool {
	int citiesNumber;
	TSPSolution first;
	
	/**
	 * Konstruktor.
	 * 
	 * @param citiesNumber broj gradova koji rješenja sadrže
	 */
	public TSPSolutionPool(int citiesNumber) {
		super();
		this.citiesNumber = citiesNumber;
	}
	
	/**
	 * Metoda iznajmljuje jedno rješenje.
	 * 
	 * @return rješenje
	 */
	public TSPSolution get() {
		if(first!=null) {
			TSPSolution s = first;
			first = (TSPSolution)first.next;
			return s;
		}
		TSPSolution s = new TSPSolution();
		s.cityIndexes = new int[citiesNumber];
		return s;
	}
	
	/**
	 * Metoda preuzima vraćeno rješenje. To rješenje
	 * kasnije se može iznajmiti, i originalni vlasnik
	 * ga više NE SMIJE koristiti.
	 * 
	 * @param sol rješenje koje se vraća
	 */
	public void release(TSPSolution sol) {
		sol.next = first;
		first = sol;
	}
}
